import gzip
import multiprocessing
from multiprocessing.pool import ThreadPool as Pool
from tileset_analyzer.entities.job_param import CompressionType
from tileset_analyzer.entities.layer_info import LayerInfo
from tileset_analyzer.readers.vector_tile.engine import VectorTile


def _processed_data(data, compressed, compression_type):
    if compressed is False:
        return data

    if compression_type == CompressionType.GZIP:
        return gzip.decompress(data)

    raise f'UNSUPPORTED COMPRESSION TYPE {compression_type}'


def get_attr(tiles, compressed, compression_type):
    attr_info = {}

    def process_tile(tile):
        if tile.z not in attr_info:
            attr_info[tile.z] = {}

        zoom_level_info = attr_info[tile.z]

        data = _processed_data(tile.data, compressed, compression_type)
        vt = VectorTile(data)
        for layer in vt.layers:
            if layer.name not in zoom_level_info:
                zoom_level_info[layer.name] = LayerInfo(layer.name, tile.z)

            layer_info = zoom_level_info[layer.name]
            for feature in layer.features:
                layer_info.add_feature(feature.attributes.get(), feature.type)

    with Pool(processes=multiprocessing.cpu_count()) as pool:
        pool.map(process_tile, tiles)

    # get all layer info's
    all_layers = []
    for zoom_level in sorted(attr_info.keys()):
        layer_dict = attr_info[zoom_level]
        for layer_name in sorted(layer_dict.keys()):
            layer_item = layer_dict[layer_name]
            all_layers.append(layer_item)
    return all_layers



from main import execute
from tileset_analyzer.entities.job_param import JobParam, CompressionType, JobAction, TileScheme


def run_mbtiles_analyze(actions):
    src_path = 'data/maptiler-osm-2017-07-03-v3.6.1-us_virginia.mbtiles'
    temp_folder = 'tileset_analyzer/static/data'
    scheme = TileScheme.TMS
    actions = actions
    compressed = True
    compression_type = CompressionType.GZIP
    job_param = JobParam(
        source=src_path,
        scheme=scheme,
        temp_folder=temp_folder,
        actions=actions,
        verbose=False,
        compressed=compressed,
        compression_type=compression_type,
        mbtiles_tbl='tiles,tile_row,tile_column,zoom_level,tile_data'
    )
    execute(job_param)


def folder_tiles_analyze(actions):
    src_path = 'data/tiles/'
    temp_folder = 'tileset_analyzer/static/data'
    scheme = TileScheme.XYZ
    actions = actions
    compressed = False
    compression_type = None
    job_param = JobParam(
        source=src_path,
        scheme=scheme,
        temp_folder=temp_folder,
        actions=actions,
        verbose=False,
        compressed=compressed,
        compression_type=compression_type,
        folder_path_scheme='{z}/{x}/{y}.pbf',
        mbtiles_tbl=None
    )
    execute(job_param)


if __name__ == "__main__":
    run_mbtiles_analyze([JobAction.PROCESS, JobAction.SERVE])
    # folder_tiles_analyze([JobAction.PROCESS, JobAction.SERVE])



#!/usr/bin/env python3
import json
import os
import re
import subprocess
import sys


def get_last_version() -> str:
    """Return the version number of the last release."""
    json_string = (
        subprocess.run(
            ["gh", "release", "view", "--json", "tagName"],
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        .stdout.decode("utf8")
        .strip()
    )

    return json.loads(json_string)["tagName"]


def bump_version(version_number: str, bump_type: str) -> str:
    """Return a copy of `version_number` with the specified version component incremented."""
    major, minor, patch = version_number.split(".")

    if bump_type == "major":
        return f"{int(major) + 1}.0.0"
    elif bump_type == "minor":
        return f"{major}.{int(minor) + 1}.0"
    elif bump_type == "patch":
        return f"{major}.{minor}.{int(patch) + 1}"
    else:
        raise ValueError(f"Invalid bump type: {bump_type}. Use 'major', 'minor', or 'patch'.")


def update_setup_py(new_version: str):
    """Update the version in setup.py file."""
    setup_file = "setup.py"

    with open(setup_file, "r") as f:
        content = f.read()

    # Replace version string
    updated_content = re.sub(
        r"version='[\d.]+',",
        f"version='{new_version}',",
        content
    )

    with open(setup_file, "w") as f:
        f.write(updated_content)

    print(f"Updated {setup_file} to version {new_version}")


def commit_and_push(version: str):
    """Commit version change and push to GitHub."""
    subprocess.run(["git", "add", "setup.py"], check=True)
    subprocess.run(
        ["git", "commit", "-m", f"chore: bump version to {version}"],
        check=True
    )
    subprocess.run(["git", "push"], check=True)
    print(f"Committed and pushed version {version}")


def create_release(bump_type: str = "patch"):
    """Create a new release on GitHub."""
    try:
        last_version_number = get_last_version()
    except subprocess.CalledProcessError as err:
        if err.stderr.decode("utf8").startswith("HTTP 404:"):
            # The project doesn't have any releases yet.
            new_version_number = "0.0.1"
        else:
            raise
    else:
        new_version_number = bump_version(last_version_number, bump_type)

    print(f"Creating {bump_type} release: {last_version_number} → {new_version_number}")

    # Update setup.py
    update_setup_py(new_version_number)

    # Commit and push
    commit_and_push(new_version_number)

    # Create GitHub release
    subprocess.run(
        ["gh", "release", "create", "--generate-notes", new_version_number],
        check=True,
    )

    print(f"✓ Release {new_version_number} created successfully!")
    print(f"  PyPI publication will start automatically via GitHub Actions")


if __name__ == "__main__":
    # Get bump type from environment variable or command line argument
    bump_type = os.environ.get("BUMP_TYPE", "patch")

    if len(sys.argv) > 1:
        bump_type = sys.argv[1]

    if bump_type not in ["major", "minor", "patch"]:
        print(f"Error: Invalid bump type '{bump_type}'")
        print("Usage: release.py [major|minor|patch]")
        sys.exit(1)

    create_release(bump_type)



import base64
from typing import List
import numpy as np
from tileset_analyzer.data_source.ds_utils import get_attr
from tileset_analyzer.data_source.mbtiles.sqllite_utils import create_connection
from tileset_analyzer.data_source.tile_source import TileSource
from tileset_analyzer.entities.job_param import JobParam, CompressionType
from tileset_analyzer.entities.layer_level_size import LayerLevelSize, TileItemSize
from tileset_analyzer.entities.level_size import LevelSize
from tileset_analyzer.entities.tile_item import TileItem
from tileset_analyzer.entities.tileset_analysis_result import LevelCount, TilesetAnalysisResult
from tileset_analyzer.data_source.mbtiles.sql_queries import SQL_COUNT_TILES, SQL_COUNT_TILES_BY_Z, \
    SQL_SUM_TILE_SIZES_BY_Z, SQL_MIN_TILE_SIZES_BY_Z, SQL_MAX_TILE_SIZES_BY_Z, SQL_AVG_TILE_SIZES_BY_Z, \
    SQL_LIST_TILE_SIZES_BY_Z, SQL_ALL_TILES
import pandas as pd
from tileset_analyzer.entities.tileset_info import TilesetInfo
import os
from pathlib import Path
from tileset_analyzer.readers.vector_tile.engine import VectorTile
import gzip
import multiprocessing
from multiprocessing.pool import ThreadPool as Pool
import sys

from tileset_analyzer.utilities.moniter import timeit


class MBTilesSource(TileSource):
    def __init__(self, job_param: JobParam):
        self.job_param = job_param
        self.conn = create_connection(job_param.source)
        self.tiles_size_z_df = None
        self.all_tile_sizes = None

    def count_tiles(self) -> int:
        cur = self.conn.cursor()
        sql = SQL_COUNT_TILES.format(
            **{
                'tiles': self.job_param.mbtiles_tbl.tiles,
                'tile_row': self.job_param.mbtiles_tbl.tile_row,
                'tile_column': self.job_param.mbtiles_tbl.tile_column,
                'zoom_level': self.job_param.mbtiles_tbl.zoom_level,
                'tile_data': self.job_param.mbtiles_tbl.tile_data
            })
        cur.execute(sql)
        count = cur.fetchone()[0]
        return count

    def count_tiles_by_z(self) -> List[LevelCount]:
        cur = self.conn.cursor()
        sql = SQL_COUNT_TILES_BY_Z.format(
            **{
                'tiles': self.job_param.mbtiles_tbl.tiles,
                'tile_row': self.job_param.mbtiles_tbl.tile_row,
                'tile_column': self.job_param.mbtiles_tbl.tile_column,
                'zoom_level': self.job_param.mbtiles_tbl.zoom_level,
                'tile_data': self.job_param.mbtiles_tbl.tile_data
            })
        cur.execute(sql)
        rows = cur.fetchall()
        result: List[LevelCount] = []
        for row in rows:
            result.append(LevelCount(row[0], row[1]))
        return result

    def _get_agg_tile_size_z(self, agg_type: str) -> List[LevelSize]:
        if agg_type == 'SUM':
            sql = SQL_SUM_TILE_SIZES_BY_Z
        elif agg_type == 'MIN':
            sql = SQL_MIN_TILE_SIZES_BY_Z
        elif agg_type == 'MAX':
            sql = SQL_MAX_TILE_SIZES_BY_Z
        elif agg_type == 'AVG':
            sql = SQL_AVG_TILE_SIZES_BY_Z
        else:
            raise 'UNKNOWN AGG TYPE'

        sql = sql.format(
            **{
                'tiles': self.job_param.mbtiles_tbl.tiles,
                'tile_row': self.job_param.mbtiles_tbl.tile_row,
                'tile_column': self.job_param.mbtiles_tbl.tile_column,
                'zoom_level': self.job_param.mbtiles_tbl.zoom_level,
                'tile_data': self.job_param.mbtiles_tbl.tile_data
            })
        cur = self.conn.cursor()
        cur.execute(sql)
        rows = cur.fetchall()
        result: List[LevelSize] = []
        for row in rows:
            result.append(LevelSize(row[0], row[1]))
        return result

    def _get_all_tiles(self) -> List[TileItem]:
        cur = self.conn.cursor()
        sql = SQL_ALL_TILES.format(
            **{
                'tiles': self.job_param.mbtiles_tbl.tiles,
                'tile_row': self.job_param.mbtiles_tbl.tile_row,
                'tile_column': self.job_param.mbtiles_tbl.tile_column,
                'zoom_level': self.job_param.mbtiles_tbl.zoom_level,
                'tile_data': self.job_param.mbtiles_tbl.tile_data
            })
        cur.execute(sql)
        rows = cur.fetchall()
        result: List[TileItem] = []
        for row in rows:
            result.append(TileItem(row[0], row[1], row[2], row[3]))
        return result

    def _set_tilesize_z_dataframe(self):
        sql = SQL_LIST_TILE_SIZES_BY_Z.format(
            **{
                'tiles': self.job_param.mbtiles_tbl.tiles,
                'tile_row': self.job_param.mbtiles_tbl.tile_row,
                'tile_column': self.job_param.mbtiles_tbl.tile_column,
                'zoom_level': self.job_param.mbtiles_tbl.zoom_level,
                'tile_data': self.job_param.mbtiles_tbl.tile_data
            })
        query = sql
        cur = self.conn.cursor()
        self.tiles_size_z_df = pd.read_sql(query, self.conn)
        cur.close()

    def _clear_tilesize_z_dataframe(self):
        self.tiles_size_z_df = None

    def _get_agg_tile_size_percentiles_z(self, percentile_type: str) -> List[LevelSize]:
        if percentile_type == '50p':
            quantile = 0.5
        elif percentile_type == '85p':
            quantile = 0.85
        elif percentile_type == '90p':
            quantile = 0.9
        elif percentile_type == '95p':
            quantile = 0.95
        elif percentile_type == '99p':
            quantile = 0.99
        else:
            raise 'UNKNOWN PERCENTILE TYPE'

        result_df = self.tiles_size_z_df.groupby('zoom_level').quantile(quantile)
        result: List[LevelSize] = []
        for row in result_df.itertuples():
            result.append(LevelSize(row[0], row[1]))
        return result

    def tiles_size_agg_min_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_z('MIN')

    def tiles_size_agg_max_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_z('MAX')

    def tiles_size_agg_avg_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_z('AVG')

    def tiles_size_agg_sum_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_z('SUM')

    def tiles_size_agg_50p_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_percentiles_z('50p')

    def tiles_size_agg_85p_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_percentiles_z('85p')

    def tiles_size_agg_90p_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_percentiles_z('90p')

    def tiles_size_agg_95p_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_percentiles_z('95p')

    def tiles_size_agg_99p_by_z(self) -> List[LevelSize]:
        return self._get_agg_tile_size_percentiles_z('99p')

    def _processed_data(self, data):
        if self.job_param.compressed is False:
            return base64.encodestring(data)

        if self.job_param.compression_type == CompressionType.GZIP:
            return gzip.decompress(data)

        raise f'UNSUPPORTED COMPRESSION TYPE {self.job_param.compression_type}'

    @timeit
    def tileset_info(self) -> TilesetInfo:
        tileset_info = TilesetInfo()
        tileset_info.set_name((self.job_param.source.split('/')[-1]).split('.')[0])
        tileset_info.set_size(os.stat(self.job_param.source).st_size)
        tileset_info.set_scheme(self.job_param.scheme)
        tileset_info.set_location(str(Path(self.job_param.source).parent.absolute()))
        tileset_info.set_ds_type('mbtiles')
        tileset_info.set_compression(self.job_param.compressed, self.job_param.compression_type)

        tiles = self._get_all_tiles()
        all_layers = get_attr(tiles, self.job_param.compressed, self.job_param.compression_type)
        tileset_info.set_layer_info(all_layers)
        return tileset_info

    @timeit
    def _preprocess_tile_layer_sizes(self):
        all_tile_sizes: dict[str, List[TileItemSize]] = {}
        tiles = self._get_all_tiles()

        def normalize_tile_sizes(total: int, layers: dict[str, int]) -> dict[str, int]:
            final = {}
            layer_sum = sum([layer_size for layer_size in layers.values()])
            for layer_name, layer_size in layers.items():
                final[layer_name] = round((layer_size / layer_sum) * total)
            return final

        def process_tile(tile):
            if tile.z not in all_tile_sizes:
                all_tile_sizes[tile.z] = []

            level_tile_sizes = all_tile_sizes[tile.z]

            data = self._processed_data(tile.data)
            size = sys.getsizeof(tile.data)
            vt = VectorTile(data)
            tile_all_layers_size = {}
            for layer in vt.layers:
                tile_layer_size = 0
                for feature in layer.features:
                    attr = feature.attributes.get()
                    geom = feature.get_geometry()
                    tile_layer_size += (sys.getsizeof(attr) + sys.getsizeof(geom))
                tile_all_layers_size[layer.name] = tile_layer_size
            tile_all_layers_size = normalize_tile_sizes(size, tile_all_layers_size)
            level_tile_sizes.append(TileItemSize(tile.x, tile.y, tile.z, size, tile_all_layers_size))

        with Pool(processes=multiprocessing.cpu_count()) as pool:
            pool.map(process_tile, tiles)

        self.all_tile_sizes = all_tile_sizes

    def _preprocess_tile_layer_sizes_clear(self):
        self.all_tile_sizes = None

    @timeit
    def tiles_size_agg_sum_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = sum([tile.size for tile in tiles])
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = 0
                    layer_sizes[layer_name] += layer_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_min_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_size == 0:
                        continue
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.min(arr))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_max_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.max(arr))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_avg_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.average(arr))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_50p_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.percentile(arr, 50))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_85p_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.percentile(arr, 85))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_90p_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.percentile(arr, 90))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_95p_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.percentile(arr, 95))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    @timeit
    def tiles_size_agg_99p_by_z_layer(self) -> List[LayerLevelSize]:
        result: List[LayerLevelSize] = []
        for z in sorted(self.all_tile_sizes.keys()):
            tiles = self.all_tile_sizes[z]
            total = 0
            layer_sizes = {}
            for item in tiles:
                for layer_name, layer_size in item.layers.items():
                    if layer_name not in layer_sizes:
                        layer_sizes[layer_name] = []
                    layer_sizes[layer_name].append(layer_size)
            for layer_name in layer_sizes.keys():
                arr = np.array(layer_sizes[layer_name])
                tile_size = int(np.percentile(arr, 99))
                layer_sizes[layer_name] = tile_size
                total += tile_size
            result.append(LayerLevelSize(z, total, layer_sizes))

        return result

    def analyze(self) -> TilesetAnalysisResult:
        result = TilesetAnalysisResult()
        result.set_count_tiles_total(self.count_tiles())
        result.set_count_tiles_by_z(self.count_tiles_by_z())
        result.set_tiles_size_agg_sum_by_z(self.tiles_size_agg_sum_by_z())
        result.set_tiles_size_agg_min_by_z(self.tiles_size_agg_min_by_z())
        result.set_tiles_size_agg_max_by_z(self.tiles_size_agg_max_by_z())
        result.set_tiles_size_agg_avg_by_z(self.tiles_size_agg_avg_by_z())

        self._set_tilesize_z_dataframe()
        result.set_tiles_size_agg_50p_by_z(self.tiles_size_agg_50p_by_z())
        result.set_tiles_size_agg_85p_by_z(self.tiles_size_agg_85p_by_z())
        result.set_tiles_size_agg_90p_by_z(self.tiles_size_agg_90p_by_z())
        result.set_tiles_size_agg_95p_by_z(self.tiles_size_agg_95p_by_z())
        result.set_tiles_size_agg_99p_by_z(self.tiles_size_agg_99p_by_z())
        self._clear_tilesize_z_dataframe()

        result.set_tileset_info(self.tileset_info())

        self._preprocess_tile_layer_sizes()
        result.set_tiles_size_agg_sum_by_z_layer(self.tiles_size_agg_sum_by_z_layer())
        result.set_tiles_size_agg_min_by_z_layer(self.tiles_size_agg_min_by_z_layer())
        result.set_tiles_size_agg_max_by_z_layer(self.tiles_size_agg_max_by_z_layer())
        result.set_tiles_size_agg_avg_by_z_layer(self.tiles_size_agg_avg_by_z_layer())
        result.set_tiles_size_agg_50p_by_z_layer(self.tiles_size_agg_50p_by_z_layer())
        result.set_tiles_size_agg_85p_by_z_layer(self.tiles_size_agg_85p_by_z_layer())
        result.set_tiles_size_agg_90p_by_z_layer(self.tiles_size_agg_90p_by_z_layer())
        result.set_tiles_size_agg_95p_by_z_layer(self.tiles_size_agg_95p_by_z_layer())
        result.set_tiles_size_agg_99p_by_z_layer(self.tiles_size_agg_99p_by_z_layer())
        self._preprocess_tile_layer_sizes_clear()

        return result



from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
import uvicorn
import os
from pathlib import Path
from starlette.datastructures import Headers
from starlette.middleware.cors import CORSMiddleware
from starlette.responses import FileResponse, Response
from tileset_analyzer.api.tile_serve_source.tile_serve_source_repository import TileServeSourceFactory
from tileset_analyzer.entities.job_param import JobParam, TileScheme

UI_PATH = f'{Path(os.path.dirname(__file__)).parent}/static/ui'

job_param: JobParam | None = None


# https://stackoverflow.com/questions/66093397/how-to-disable-starlette-static-files-caching
class NoCacheStaticFiles(StaticFiles):
    def is_not_modified(
            self, response_headers: Headers, request_headers: Headers
    ) -> bool:
        # your own cache rules goes here...
        return False


def start_api(_job_param: JobParam):
    global job_param
    job_param = _job_param

    app = FastAPI()
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    app.mount("/api", app=NoCacheStaticFiles(directory=job_param.temp_folder), name="api")

    @app.get("/tileset/{z}/{x}/{y}.mvt")
    async def get_tile(z: int, x: int, y: int):
        data = None
        if job_param.scheme == TileScheme.TMS:
            y = (1 << z) - 1 - y
            data = TileServeSourceFactory.get_tile(job_param, z, y, x)
        elif job_param.scheme == TileScheme.XYZ:
            data = TileServeSourceFactory.get_tile(job_param, z, x, y)

        if not data:
            return Response(status_code=404)

        if job_param.compressed:
            headers = {
                "content-encoding": job_param.compression_type,
                "content-type": "application/vnd.mapbox-vector-tile",
                "Cache-Control": 'no-cache, no-store'
            }
        else:
            headers = {
                "content-type": "application/vnd.mapbox-vector-tile",
                "Cache-Control": 'no-cache, no-store'
            }
        return Response(content=data, headers=headers)

    @app.get("/")
    async def index():
        return FileResponse(f'{UI_PATH}/index.html')

    app.mount("/static", app=NoCacheStaticFiles(directory=f'{UI_PATH}/static'), name="ui")

    @app.get("/{file_name}.{file_path}")
    async def root_files(file_name: str, file_path: str):
        return FileResponse(f'{UI_PATH}/{file_name}.{file_path}')

    @app.get("/{path_name:path}")
    async def index_react_path():
        return FileResponse(f'{UI_PATH}/index.html')
    
    uvicorn.run(app, host="0.0.0.0", port=8080)
    



from setuptools import setup, find_packages
import re
from pathlib import Path

# Read version from __init__.py
def get_version():
    init_file = Path(__file__).parent / "tileset_analyzer" / "__init__.py"
    content = init_file.read_text()
    match = re.search(r'__version__\s*=\s*["\']([^"\']+)["\']', content)
    if match:
        return match.group(1)
    raise RuntimeError("Unable to find version string.")

CLASSIFIERS = [
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.12",
]

setup(
    name='tileset_analyzer',
    version=get_version(),
    url='https://github.com/geoyogesh/tileset_analyzer',
    license='MIT',
    author='Yogesh Dhanapal',
    author_email='geoyogesh@gmail.com',
    entry_points={"console_scripts": ["tileset_analyzer = tileset_analyzer.main:cli"]},
    description='Analyze vector Tileset',
    python_requires=">=3.10, <4",
    packages=find_packages(),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    zip_safe=False,
    classifiers=CLASSIFIERS,
    include_package_data=True,
    install_requires=["fastapi", "uvicorn[standard]", "pandas", "protobuf<=3.20.3", "parse"]
)



from tileset_analyzer.api.main_api import start_api
from tileset_analyzer.data_source.tile_source_factory import TilesetSourceFactory
from tileset_analyzer.entities.job_param import JobParam, CompressionType, TileScheme, JobAction
from tileset_analyzer.utilities.moniter import timeit
from tileset_analyzer.utils.json_utils import write_json_file
import sys
import os
import argparse


def execute(job_param: JobParam):
    print('started...')
    print('src_path:', job_param.source)
    print('scheme:', job_param.scheme)
    print('compressed:', job_param.compressed)
    print('compression_type:', job_param.compression_type)
    print('temp_folder:', job_param.temp_folder)
    print('actions', job_param.actions)
    print('folder_path_scheme', job_param.folder_path_scheme)
    print('mbtiles_tbl', job_param.mbtiles_tbl)
    print('verbose', job_param.verbose)

    if 'process' in job_param.actions:
        process_job(job_param)

    if 'serve' in job_param.actions:
        print('Web UI started')
        start_api(job_param)
        print('Web UI stopped')

    print('completed')


@timeit
def process_job(job_param: JobParam):
    print('processing started')
    data_source = TilesetSourceFactory.get_tileset_source(job_param)
    result = data_source.analyze()
    output_json = os.path.join(job_param.temp_folder, 'analysis_result.json')
    write_json_file(result.get_json(), output_json)
    print('processing completed')


def get_arg(param):
    source_index = sys.argv.index(param)
    val = sys.argv[source_index + 1]
    return val


def cli():
    parser = argparse.ArgumentParser(prog='tileset_analyzer')
    parser.add_argument('--source', help='source', required=True)
    parser.add_argument('--scheme', help='scheme', default=TileScheme.XYZ)
    parser.add_argument('--compressed', help='compressed', default=False, action=argparse.BooleanOptionalAction)
    parser.add_argument('--compression_type', help='compression_type', default=CompressionType.GZIP)
    parser.add_argument('--temp_folder', help='temp_folder', required=True)
    parser.add_argument('--actions', help='actions', default=[JobAction.PROCESS, JobAction.SERVE])
    parser.add_argument("--verbose", help="increase output verbosity", action="store_false")
    parser.add_argument('--folder_path_scheme', help='folder path scheme', default=None)
    parser.add_argument('--mbtiles_tbl', help='mbtiles tbl params', default='tiles,tile_row,tile_column,zoom_level,tile_data')

    args = parser.parse_args()
    actions = args.actions.split(',')

    job_param = JobParam(
        source=args.source,
        scheme=args.scheme,
        temp_folder=args.temp_folder,
        actions=actions,
        verbose=args.verbose,
        compressed=args.compressed,
        compression_type=args.compression_type,
        folder_path_scheme=args.folder_path_scheme,
        mbtiles_tbl=args.mbtiles_tbl)
    execute(job_param)


'''
if __name__ == "__main__":
   cli()
'''